==> glite-apel-sge-2.0.8-4.src.tar.gz: glite-apel-sge-2.0.8/src/org/glite/apel/sge/EventLogParser.java <==
/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.apel.sge;



import org.apache.log4j.Logger;

import org.glite.apel.core.Log;

import org.glite.apel.core.Time;

import org.glite.apel.core.TimeInterval;

import org.glite.apel.core.log.type.EventRecord;

import org.glite.apel.core.log.type.EventRecordIterator;



import java.io.BufferedReader;

import java.io.File;

import java.io.FileInputStream;

import java.io.IOException;

import java.io.InputStreamReader;

import java.text.SimpleDateFormat;

import java.util.Calendar;

import java.util.Date;

import java.util.TimeZone;

import java.util.Vector;

import java.util.regex.Pattern;

import java.util.zip.GZIPInputStream;





/**

 * A file containing a SGE event log

 */

class EventLogParser extends File {

    

    /**

     * Using log4j logging tool.

     */

    protected static final Logger LOG = 

        Logger.getLogger(EventLogParser.class.getName());

    

    /**

     * Buffer which reads-in data from the log file.

     */

    protected BufferedReader m_reader;

    

    /**

     * Holds all event records read-in from the event log file.

     */

    protected Vector m_eventRecords;

    

    /** 

     * The format used to encode the current date. 

     */

    private static final String EVENT_DATE_FORMAT = "yyyyMMdd";

    

    /** 

     * UTC constant used when converting the current date. 

     */

    private static final String UTC = "UTC";

    

    /**

     * Formatter converts the current date into the same

     * format used by the event log file.

     */

    private SimpleDateFormat m_dateFormatter;

    

    /**

     * The timezone to be used when interpreting time stamps 

     * parsed from the log files.

     */

    private String m_timezone;

    

    /**

     * Pattern used to chop up a line of data that matches an

     * EventRecord into a convenient list.

     */

    private Pattern m_delimiter;

    

    /**

     * Pattern used to match a pbs event record in the log file.

     */

    private Pattern m_endEventPattern;



    /**

     * Pattern used to match a sge event record in the log file.

     */

    private Pattern m_startEventPattern;



    /**

     * SGE logs don't report the batch submit host: This is added by means of a 

     * <SubmitHost>value</SumitHost> token in APEL Event log config file.

     */

     private String m_submitHost;

 

     /**
      * The file object representing this logfile
      */
     private File m_file;
    

    /**

     * Creates a new event log file.

     */

    EventLogParser(File file, String timezone, String submitHost) {

        super(file.getPath());

        m_eventRecords = new Vector();

        m_submitHost = submitHost;

        m_file = file;


        



        m_timezone = timezone;

        //m_dateFormatter = new SimpleDateFormat(EVENT_DATE_FORMAT);

        //m_dateFormatter.setTimeZone(TimeZone.getTimeZone(UTC));

        m_delimiter = Pattern.compile(":");

        String prefixString = "^\\S*:{1}";

        m_endEventPattern = Pattern.compile(prefixString + ".*");

        //String fixString = "^MyType = \"Job\"$";

        //m_startEventPattern = Pattern.compile(fixString);

    }

    public void openLogFile() {
        try {

            FileInputStream fileInputStream = new FileInputStream(m_file);

            if (getName().matches("^.*\\.gz$")) {

                m_reader = new BufferedReader(new InputStreamReader(

                    new GZIPInputStream(fileInputStream)));

            } else {

                m_reader = new BufferedReader(new InputStreamReader(

                    fileInputStream));

            }

        } catch (Exception e) {

            Log.warn(LOG, "Exception opening file " + m_file.getPath(), e);

        }
    }
    

    /**

     * Returns true if the file name matches today's date (ie the log

     * is currently being used).

     */

    boolean isActiveLog() {

        SimpleDateFormat m_dateFormatter = new SimpleDateFormat(EVENT_DATE_FORMAT);

        m_dateFormatter.setTimeZone(TimeZone.getDefault());

        Date currentDate = Calendar.getInstance().getTime();

        String formattedDate = m_dateFormatter.format(currentDate);

        ////////////////////////////////////// 

        // No clear requirmemnts from the community

        // Log rotated file accounting.20060407 

        // or just a file called accounting (ALWAYS REURN TRUE)

        //////////////////////////////////////

        //String EventLogPrefix = "accounting.";

        //String CurrentLog = EventLogPrefix + formattedDate;

        //return CurrentLog.equals(getName());

        //Returns True always as SGE Event log is a history file that is always valid.

        return true;

    }

    

    /**

     * Parses the log file and returns an iterator over all the 

     * event records found within the file.

     */

    public void parse() {

        Log.info(LOG, "Processing " + getFullPath());

        String line = null;

        try {

            while ((line = m_reader.readLine()) != null) {

                if (m_endEventPattern.matcher(line).matches()) {

                    EventRecord record = createEventRecord(line);

                    m_eventRecords.add(record);

                }

            }

        } catch (IOException e) {

            Log.warn(LOG, "Exception reading file " + getName(), e);

        }

    }

    

    /**

     * Returns a new EventRecord derived from the line of data 

     * (parsed from the event log). 

     */

    EventRecord createEventRecord(String line) {

        EventRecord record = new EventRecord();

        String[] fields = m_delimiter.split(line);

        record.setSubmitHost(m_submitHost);

        String jobNumber=(String)fields[5];

        record.setJobName(jobNumber);



        // Owner

        String owner=(String)fields[3];

        record.setLocalUserID(owner);

        // Group

        String group=(String)fields[2];

        record.setLocalUserGroup(group);

         



        TimeInterval interval;

    

	Double userTime=new Double((String)fields[14]);

	Double systemTime=new Double((String)fields[15]);	

        // Set CpuTime

        // NB we want to round cpuTime to nearest whole

        Double cpuTime=userTime + systemTime;

        double cpu = Math.round(cpuTime.doubleValue()); // 

        //System.out.println("Math.round(cpu)= " + cpu);

        cpuTime=new Double(cpu);

        record.setCpuDurationSeconds(Integer.toString(cpuTime.intValue()));

        //System.out.println("CpuDurationSeconds= " + Integer.toString(cpuTime.intValue()));

        //System.out.println("CPU= " + fields[36]);



        // Convert cpuDurationSeconds into hh::mm:ss

        String cpu_string = secondsToString(cpuTime.doubleValue());

        //System.out.println("CpuDurationSeconds into hh::mm:ss = " + cpu_string);

        interval = new TimeInterval(cpu_string);

        //System.out.println("CpuDuration= " + interval.getIso8601());

        record.setCpuDuration(interval.getIso8601());



        // Set wallDurationSeconds

        Double ru_wallclock = new Double((String)fields[13]);

        //System.out.println("ru_wallclock  = " + Integer.toString(ru_wallclock.intValue()));

        record.setWallDurationSeconds(Integer.toString(ru_wallclock.intValue()));

        

        // Convert wallDurationSeconds into hh::mm:ss

        String wds_string = secondsToString(ru_wallclock.doubleValue());

        //System.out.println("wallDurationSeconds into hh::mm:ss = " + wds_string);

        interval = new TimeInterval(wds_string);

        record.setWallDuration(interval.getIso8601());

 



        /* 

        start/stop times defined in epoch seconds which must be adjusted

        into UTC format. 

        */       

        ////////////////////////////////////////////////////////////////////////////

        // TIMEZONE

        ////////////////////////////////////////////////////////////////////////////

        TimeZone convertedTimezone = null;

        TimeZone timezone = TimeZone.getTimeZone(m_timezone);



        /**

         *Start/Stop times defined in epoch seconds which must be adjusted into UTC format.

         */



        Time time = new Time((String)fields[10],timezone);

        record.setStopTimeEpoch(time.getEpoch());

        record.setStopTime(time.getIso8601());

        String stopTimeUtc = time.getIso8601Utc();

        record.setStopTimeUtc(stopTimeUtc);

        record.setEventDate(stopTimeUtc.substring(0, 10));

        record.setEventTime(stopTimeUtc.substring(11, 19));

        //System.out.println("Stop Time Epoch      = " + time.getEpoch());

        //System.out.println("Stop Time Iso8601Utc = " + time.getIso8601Utc());

        //System.out.println("Stop Time Iso8601    = " + time.getIso8601());

        //System.out.println("EventDate   = " + time.getIso8601Utc().substring(0,10));

        //System.out.println("EventTime   = " + time.getIso8601Utc().substring(11,19));

       

        time = new Time((String)fields[9], timezone);

        record.setStartTimeEpoch(time.getEpoch());

        record.setStartTime(time.getIso8601());

        record.setStartTimeUtc(time.getIso8601Utc());

        record.setEventDate(stopTimeUtc.substring(0, 10));

        record.setEventTime(stopTimeUtc.substring(11, 19));

        //System.out.println("Start Time Epoch     = " + time.getEpoch());

        //System.out.println("Stop Time Iso8601Utc = " + time.getIso8601Utc());

        //System.out.println("Start Time Iso8601   = " + time.getIso8601());      



        return record;

    }

    

    /**

     * Remove the everything from the start of the string to the 

     * specified character.

     *

     * @param string string to strip.

     * @param delimiter character to look for.

     * @return string with leading text stripped off.

     */

    private String strip(String string, char delimiter) {

        int index = string.indexOf(delimiter);

        if (index != -1) {

            return string.substring(index + 1);

        }

        return string;

    }

    

    /**

     * Closes down the log file object. This is called once all meaningful 

     * data has been read from the file.

     */

    public void close() {

        if (m_reader != null) {

            try {

                m_reader.close();

            } catch (IOException e) {

                Log.warn(LOG, "Exception closing file " + getName(), e);

            }

        }

    }

    

    /**

     * Return canonical path, with built-in exception handling.

     * If an exception is thrown only the relative path is returned.

     */

    public String getFullPath() {

        String path;

        try {

            path = getCanonicalPath();

        } catch (Exception e) {

            Log.warn(LOG, "Exception trying to get filename", e);

            path = getName();

        }

        return path;

    }

    

    /**

     * Returns an iterator of all the parsed event records.

     */

    public EventRecordIterator getRecords() {

        return new EventRecordIterator(m_eventRecords.iterator());

    }

    



   /**

     * Converts time in seconds to a <code>String</code> in the

     * format HH:mm:ss.

     */

    private String secondsToString(double time) {

     int seconds = (int) (time % 60);

     int minutes = (int) ((time/60) % 60);

     int hours = (int) ((time/3600) % 24);

     int days = (int) ((time/3600)/24);

     hours += days * 24;

     String secondsStr = (seconds < 10 ? "0" : "") + seconds;

     String minutesStr = (minutes < 10 ? "0" : "") + minutes;

     String hoursStr = (hours < 10 ? "0" : "") + hours;

     return new String(hoursStr + ":" + minutesStr + ":" + secondsStr);

    }





}

==> glite-apel-sge-2.0.8-5.src.tar.gz: glite-apel-sge-2.0.8/src/org/glite/apel/sge/EventLogParser.java <==
/*
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010.
 * See http://www.eu-egee.org/partners/ for details on the
 * copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.glite.apel.sge;



import org.apache.log4j.Logger;

import org.glite.apel.core.Log;

import org.glite.apel.core.Time;

import org.glite.apel.core.TimeInterval;

import org.glite.apel.core.log.type.EventRecord;

import org.glite.apel.core.log.type.EventRecordIterator;



import java.io.BufferedReader;

import java.io.File;

import java.io.FileInputStream;

import java.io.IOException;

import java.io.InputStreamReader;

import java.text.SimpleDateFormat;

import java.util.Calendar;

import java.util.Date;

import java.util.TimeZone;

import java.util.Vector;

import java.util.regex.Pattern;

import java.util.zip.GZIPInputStream;





/**

 * A file containing a SGE event log

 */

class EventLogParser extends File {

    

    /**

     * Using log4j logging tool.

     */

    protected static final Logger LOG = 

        Logger.getLogger(EventLogParser.class.getName());

    

    /**

     * Buffer which reads-in data from the log file.

     */

    protected BufferedReader m_reader;

    

    /**

     * Holds all event records read-in from the event log file.

     */

    protected Vector m_eventRecords;

    

    /** 

     * The format used to encode the current date. 

     */

    private static final String EVENT_DATE_FORMAT = "yyyyMMdd";

    

    /** 

     * UTC constant used when converting the current date. 

     */

    private static final String UTC = "UTC";

    

    /**

     * Formatter converts the current date into the same

     * format used by the event log file.

     */

    private SimpleDateFormat m_dateFormatter;

    

    /**

     * The timezone to be used when interpreting time stamps 

     * parsed from the log files.

     */

    private String m_timezone;

    

    /**

     * Pattern used to chop up a line of data that matches an

     * EventRecord into a convenient list.

     */

    private Pattern m_delimiter;

    

    /**

     * Pattern used to match a pbs event record in the log file.

     */

    private Pattern m_endEventPattern;



    /**

     * Pattern used to match a sge event record in the log file.

     */

    private Pattern m_startEventPattern;



    /**

     * SGE logs don't report the batch submit host: This is added by means of a 

     * <SubmitHost>value</SumitHost> token in APEL Event log config file.

     */

     private String m_submitHost;

 

     /**
      * The file object representing this logfile
      */
     private File m_file;
    

    /**

     * Creates a new event log file.

     */

    EventLogParser(File file, String timezone, String submitHost) {

        super(file.getPath());

        m_eventRecords = new Vector();

        m_submitHost = submitHost;

        m_file = file;


        



        m_timezone = timezone;

        //m_dateFormatter = new SimpleDateFormat(EVENT_DATE_FORMAT);

        //m_dateFormatter.setTimeZone(TimeZone.getTimeZone(UTC));

        m_delimiter = Pattern.compile(":");

        String prefixString = "^\\S*:{1}";

        m_endEventPattern = Pattern.compile(prefixString + ".*");

        //String fixString = "^MyType = \"Job\"$";

        //m_startEventPattern = Pattern.compile(fixString);

    }

    public void openLogFile() {
        try {

            FileInputStream fileInputStream = new FileInputStream(m_file);

            if (getName().matches("^.*\\.gz$")) {

                m_reader = new BufferedReader(new InputStreamReader(

                    new GZIPInputStream(fileInputStream)));

            } else {

                m_reader = new BufferedReader(new InputStreamReader(

                    fileInputStream));

            }

        } catch (Exception e) {

            Log.warn(LOG, "Exception opening file " + m_file.getPath(), e);

        }
    }
    

    /**

     * Returns true if the file name matches today's date (ie the log

     * is currently being used).

     */

    boolean isActiveLog() {

        SimpleDateFormat m_dateFormatter = new SimpleDateFormat(EVENT_DATE_FORMAT);

        m_dateFormatter.setTimeZone(TimeZone.getDefault());

        Date currentDate = Calendar.getInstance().getTime();

        String formattedDate = m_dateFormatter.format(currentDate);

        ////////////////////////////////////// 

        // No clear requirmemnts from the community

        // Log rotated file accounting.20060407 

        // or just a file called accounting (ALWAYS REURN TRUE)

        //////////////////////////////////////

        String EventLogPrefix = "accounting.";

        String CurrentLog = EventLogPrefix + formattedDate;

        if (CurrentLog.equals(getName()) || getName().equals("accounting")){
            return true;
        }

        //Returns True always as SGE Event log is a history file that is always valid.

        return false;

    }

    

    /**

     * Parses the log file and returns an iterator over all the 

     * event records found within the file.

     */

    public void parse() {

        Log.info(LOG, "Processing " + getFullPath());

        String line = null;

        try {

            while ((line = m_reader.readLine()) != null) {

                if (m_endEventPattern.matcher(line).matches()) {

                    EventRecord record = createEventRecord(line);

                    m_eventRecords.add(record);

                }

            }

        } catch (IOException e) {

            Log.warn(LOG, "Exception reading file " + getName(), e);

        }

    }

    

    /**

     * Returns a new EventRecord derived from the line of data 

     * (parsed from the event log). 

     */

    EventRecord createEventRecord(String line) {

        EventRecord record = new EventRecord();

        String[] fields = m_delimiter.split(line);

        record.setSubmitHost(m_submitHost);

        String jobNumber=(String)fields[5];

        record.setJobName(jobNumber);



        // Owner

        String owner=(String)fields[3];

        record.setLocalUserID(owner);

        // Group

        String group=(String)fields[2];

        record.setLocalUserGroup(group);

         



        TimeInterval interval;

    

	Double userTime=new Double((String)fields[14]);

	Double systemTime=new Double((String)fields[15]);	

        // Set CpuTime

        // NB we want to round cpuTime to nearest whole

        Double cpuTime=userTime + systemTime;

        double cpu = Math.round(cpuTime.doubleValue()); // 

        //System.out.println("Math.round(cpu)= " + cpu);

        cpuTime=new Double(cpu);

        record.setCpuDurationSeconds(Integer.toString(cpuTime.intValue()));

        //System.out.println("CpuDurationSeconds= " + Integer.toString(cpuTime.intValue()));

        //System.out.println("CPU= " + fields[36]);



        // Convert cpuDurationSeconds into hh::mm:ss

        String cpu_string = secondsToString(cpuTime.doubleValue());

        //System.out.println("CpuDurationSeconds into hh::mm:ss = " + cpu_string);

        interval = new TimeInterval(cpu_string);

        //System.out.println("CpuDuration= " + interval.getIso8601());

        record.setCpuDuration(interval.getIso8601());



        // Set wallDurationSeconds

        Double ru_wallclock = new Double((String)fields[13]);

        //System.out.println("ru_wallclock  = " + Integer.toString(ru_wallclock.intValue()));

        record.setWallDurationSeconds(Integer.toString(ru_wallclock.intValue()));

        

        // Convert wallDurationSeconds into hh::mm:ss

        String wds_string = secondsToString(ru_wallclock.doubleValue());

        //System.out.println("wallDurationSeconds into hh::mm:ss = " + wds_string);

        interval = new TimeInterval(wds_string);

        record.setWallDuration(interval.getIso8601());

 



        /* 

        start/stop times defined in epoch seconds which must be adjusted

        into UTC format. 

        */       

        ////////////////////////////////////////////////////////////////////////////

        // TIMEZONE

        ////////////////////////////////////////////////////////////////////////////

        TimeZone convertedTimezone = null;

        TimeZone timezone = TimeZone.getTimeZone(m_timezone);



        /**

         *Start/Stop times defined in epoch seconds which must be adjusted into UTC format.

         */



        Time time = new Time((String)fields[10],timezone);

        record.setStopTimeEpoch(time.getEpoch());

        record.setStopTime(time.getIso8601());

        String stopTimeUtc = time.getIso8601Utc();

        record.setStopTimeUtc(stopTimeUtc);

        record.setEventDate(stopTimeUtc.substring(0, 10));

        record.setEventTime(stopTimeUtc.substring(11, 19));

        //System.out.println("Stop Time Epoch      = " + time.getEpoch());

        //System.out.println("Stop Time Iso8601Utc = " + time.getIso8601Utc());

        //System.out.println("Stop Time Iso8601    = " + time.getIso8601());

        //System.out.println("EventDate   = " + time.getIso8601Utc().substring(0,10));

        //System.out.println("EventTime   = " + time.getIso8601Utc().substring(11,19));

       

        time = new Time((String)fields[9], timezone);

        record.setStartTimeEpoch(time.getEpoch());

        record.setStartTime(time.getIso8601());

        record.setStartTimeUtc(time.getIso8601Utc());

        record.setEventDate(stopTimeUtc.substring(0, 10));

        record.setEventTime(stopTimeUtc.substring(11, 19));

        //System.out.println("Start Time Epoch     = " + time.getEpoch());

        //System.out.println("Stop Time Iso8601Utc = " + time.getIso8601Utc());

        //System.out.println("Start Time Iso8601   = " + time.getIso8601());      



        return record;

    }

    

    /**

     * Remove the everything from the start of the string to the 

     * specified character.

     *

     * @param string string to strip.

     * @param delimiter character to look for.

     * @return string with leading text stripped off.

     */

    private String strip(String string, char delimiter) {

        int index = string.indexOf(delimiter);

        if (index != -1) {

            return string.substring(index + 1);

        }

        return string;

    }

    

    /**

     * Closes down the log file object. This is called once all meaningful 

     * data has been read from the file.

     */

    public void close() {

        if (m_reader != null) {

            try {

                m_reader.close();

            } catch (IOException e) {

                Log.warn(LOG, "Exception closing file " + getName(), e);

            }

        }

    }

    

    /**

     * Return canonical path, with built-in exception handling.

     * If an exception is thrown only the relative path is returned.

     */

    public String getFullPath() {

        String path;

        try {

            path = getCanonicalPath();

        } catch (Exception e) {

            Log.warn(LOG, "Exception trying to get filename", e);

            path = getName();

        }

        return path;

    }

    

    /**

     * Returns an iterator of all the parsed event records.

     */

    public EventRecordIterator getRecords() {

        return new EventRecordIterator(m_eventRecords.iterator());

    }

    



   /**

     * Converts time in seconds to a <code>String</code> in the

     * format HH:mm:ss.

     */

    private String secondsToString(double time) {

     int seconds = (int) (time % 60);

     int minutes = (int) ((time/60) % 60);

     int hours = (int) ((time/3600) % 24);

     int days = (int) ((time/3600)/24);

     hours += days * 24;

     String secondsStr = (seconds < 10 ? "0" : "") + seconds;

     String minutesStr = (minutes < 10 ? "0" : "") + minutes;

     String hoursStr = (hours < 10 ? "0" : "") + hours;

     return new String(hoursStr + ":" + minutesStr + ":" + secondsStr);

    }





}
